"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const spec_differ_1 = require("./openapi-diff/spec-differ");
class OpenApiDiff {
    constructor(contentLoader, resultReporter) {
        this.contentLoader = contentLoader;
        this.resultReporter = resultReporter;
    }
    diffPaths(sourceSpecPath, destinationSpecPath) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const loadedSpecs = yield this.loadSpecs(sourceSpecPath, destinationSpecPath);
                const diffOutcome = yield this.diffSpecs(loadedSpecs);
                this.resultReporter.reportOutcome(diffOutcome);
                if (diffOutcome.breakingDifferencesFound) {
                    return Promise.reject(new Error('Breaking differences found'));
                }
            }
            catch (error) {
                this.resultReporter.reportError(error);
                throw error;
            }
        });
    }
    diffSpecs(options) {
        return spec_differ_1.SpecDiffer.diffSpecs(options);
    }
    loadSpecs(sourceSpecPath, destinationSpecPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const whenSourceSpec = this.contentLoader.load(sourceSpecPath);
            const whenDestinationSpec = this.contentLoader.load(destinationSpecPath);
            const [sourceSpec, destinationSpec] = yield Promise.all([whenSourceSpec, whenDestinationSpec]);
            return {
                destinationSpec: { location: destinationSpecPath, content: destinationSpec },
                sourceSpec: { location: sourceSpecPath, content: sourceSpec }
            };
        });
    }
}
exports.OpenApiDiff = OpenApiDiff;
