"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const diff_classifier_1 = require("./spec-differ/diff-classifier");
const diff_finder_1 = require("./spec-differ/diff-finder");
const spec_deserialiser_1 = require("./spec-differ/spec-deserialiser");
const spec_parser_1 = require("./spec-parser");
class SpecDiffer {
    static diffSpecs({ sourceSpec, destinationSpec }) {
        return __awaiter(this, void 0, void 0, function* () {
            const loadedSourceSpec = spec_deserialiser_1.SpecDeserialiser.load(sourceSpec);
            const loadedDestinationSpec = spec_deserialiser_1.SpecDeserialiser.load(destinationSpec);
            const parsedSourceSpec = yield spec_parser_1.specParser.parse(loadedSourceSpec, sourceSpec.location);
            const parsedDestinationSpec = yield spec_parser_1.specParser.parse(loadedDestinationSpec, destinationSpec.location);
            const differences = yield diff_finder_1.DiffFinder.findDifferences({
                destinationSpec: parsedDestinationSpec,
                sourceSpec: parsedSourceSpec
            });
            const classifiedDifferences = diff_classifier_1.DiffClassifier.classifyDifferences(differences);
            const sourceSpecDetails = {
                format: parsedSourceSpec.format,
                location: sourceSpec.location
            };
            const destinationSpecDetails = {
                format: parsedDestinationSpec.format,
                location: destinationSpec.location
            };
            return this.generateDiffOutcome(classifiedDifferences, sourceSpecDetails, destinationSpecDetails);
        });
    }
    static generateDiffOutcome(classifiedDifferences, sourceSpecDetails, destinationSpecDetails) {
        const breakingDifferencesFound = classifiedDifferences.breakingDifferences.length > 0;
        if (breakingDifferencesFound) {
            return {
                breakingDifferences: classifiedDifferences.breakingDifferences,
                breakingDifferencesFound,
                destinationSpecDetails,
                nonBreakingDifferences: classifiedDifferences.nonBreakingDifferences,
                sourceSpecDetails,
                unclassifiedDifferences: classifiedDifferences.unclassifiedDifferences
            };
        }
        else {
            return {
                breakingDifferencesFound,
                destinationSpecDetails,
                nonBreakingDifferences: classifiedDifferences.nonBreakingDifferences,
                sourceSpecDetails,
                unclassifiedDifferences: classifiedDifferences.unclassifiedDifferences
            };
        }
    }
}
exports.SpecDiffer = SpecDiffer;
