"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const create_difference_1 = require("./create-difference");
const normalize_path_1 = require("./normalize-path");
const findAddedPathDifferences = (sourcePathItems, destinationPathItems) => {
    const sourcePathNames = getPathNames(sourcePathItems);
    return destinationPathItems
        .filter((pathItem) => !_.includes(sourcePathNames, pathItem.pathName))
        .map((pathItem) => create_difference_1.createDifference({
        action: 'add',
        destinationObject: pathItem.originalValue,
        propertyName: 'path'
    }));
};
const findRemovedPathDifferences = (sourcePathItems, destinationPathItems) => {
    const destinationPathNames = getPathNames(destinationPathItems);
    return sourcePathItems
        .filter((pathItem) => !_.includes(destinationPathNames, pathItem.pathName))
        .map((pathItem) => create_difference_1.createDifference({
        action: 'remove',
        propertyName: 'path',
        sourceObject: pathItem.originalValue
    }));
};
const getPathNames = (pathItems) => pathItems.map((pathItem) => pathItem.pathName);
const normalizePathItems = (pathItems) => pathItems.map((pathItem) => (Object.assign({}, pathItem, { pathName: normalize_path_1.normalizePath(pathItem.pathName) })));
exports.findDiffsInPaths = (sourcePathItems, destinationPathItems) => {
    const normalisedSourcePathItems = normalizePathItems(sourcePathItems);
    const normalisedDestinationPathItems = normalizePathItems(destinationPathItems);
    const addedPaths = findAddedPathDifferences(normalisedSourcePathItems, normalisedDestinationPathItems);
    const removedPaths = findRemovedPathDifferences(normalisedSourcePathItems, normalisedDestinationPathItems);
    return [...addedPaths, ...removedPaths];
};
