"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const create_difference_1 = require("./create-difference");
const findDiffsInProperty = (sourceObject, destinationObject, propertyName) => {
    const additionDiffs = findAdditionDiffsInProperty(sourceObject, destinationObject, propertyName);
    const deletionDiffs = findDeletionDiffsInProperty(sourceObject, destinationObject, propertyName);
    const editionDiffs = findEditionDiffsInProperty(sourceObject, destinationObject, propertyName);
    return _.concat([], additionDiffs, deletionDiffs, editionDiffs);
};
const findAdditionDiffsInProperty = (sourceObject, destinationObject, propertyName) => {
    const isAddition = isUndefinedDeep(sourceObject) && isDefinedDeep(destinationObject);
    if (isAddition) {
        return [create_difference_1.createDifference({ sourceObject, destinationObject, propertyName, action: 'add' })];
    }
    return [];
};
const findDeletionDiffsInProperty = (sourceObject, destinationObject, propertyName) => {
    const isDeletion = isDefinedDeep(sourceObject) && isUndefinedDeep(destinationObject);
    if (isDeletion) {
        return [create_difference_1.createDifference({ sourceObject, destinationObject, propertyName, action: 'remove' })];
    }
    return [];
};
const findEditionDiffsInProperty = (sourceObject, destinationObject, propertyName) => {
    const isEdition = isDefinedDeep(sourceObject)
        && isDefinedDeep(destinationObject) && !_.isEqual(sourceObject.value, destinationObject.value);
    if (isEdition) {
        return [
            create_difference_1.createDifference({ sourceObject, destinationObject, propertyName, action: 'add' }),
            create_difference_1.createDifference({ sourceObject, destinationObject, propertyName, action: 'remove' })
        ];
    }
    return [];
};
const isUndefinedDeep = (objectWithValue) => {
    return _.isUndefined(objectWithValue) || _.isUndefined(objectWithValue.value);
};
const isDefinedDeep = (objectWithValue) => {
    return isDefined(objectWithValue) && isDefined(objectWithValue.value);
};
const isDefined = (target) => {
    return !_.isUndefined(target);
};
exports.findDiffsInXProperties = (sourceParsedXProperties, destinationParsedXProperties, xPropertyContainerName) => {
    const xPropertyUniqueNames = _(_.keys(sourceParsedXProperties))
        .concat(_.keys(destinationParsedXProperties))
        .uniq()
        .value();
    const xPropertyDiffs = _(xPropertyUniqueNames)
        .map((xPropertyName) => {
        return findDiffsInProperty(sourceParsedXProperties[xPropertyName], destinationParsedXProperties[xPropertyName], `${xPropertyContainerName}.${xPropertyName}`);
    })
        .flatten()
        .value();
    return xPropertyDiffs;
};
